@@ -1,4 +1,7 @@
-# coding=utf-8
+# This file is part of the Open Data Cube, see https://opendatacube.org for more information
+#
+# Copyright (c) 2015-2020 ODC Contributors
+# SPDX-License-Identifier: Apache-2.0
 """
 User configuration.
 """
@@ -191,20 +194,32 @@ def parse_connect_url(url: str) -> Dict[str, str]:
 
 def parse_env_params() -> Dict[str, str]:
     """
+    - Read DATACUBE_IAM_* environment variables.
     - Extract parameters from DATACUBE_DB_URL if present
     - Else look for DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE
     - Return {} otherwise
     """
+    # Handle environment vars that cannot fit in the DB URL
+    non_url_params = {}
+    iam_auth = os.environ.get('DATACUBE_IAM_AUTHENTICATION')
+    if iam_auth is not None and iam_auth.lower() in ['y', 'yes']:
+        non_url_params["iam_authentication"] = True
+        iam_auth_timeout = os.environ.get('DATACUBE_IAM_TIMEOUT')
+        if iam_auth_timeout:
+            non_url_params["iam_timeout"] = int(iam_auth_timeout)
 
+    # Handle environment vars that may fit in the DB URL
     db_url = os.environ.get('DATACUBE_DB_URL', None)
     if db_url is not None:
-        return parse_connect_url(db_url)
-
-    params = {k: os.environ.get('DB_{}'.format(k.upper()), None)
-              for k in DB_KEYS}
-    return {k: v
-            for k, v in params.items()
-            if v is not None and v != ""}
+        params = parse_connect_url(db_url)
+    else:
+        raw_params = {k: os.environ.get('DB_{}'.format(k.upper()), None)
+                      for k in DB_KEYS}
+        params = {k: v
+                  for k, v in raw_params.items()
+                  if v is not None and v != ""}
+    params.update(non_url_params)
+    return params
 
 
 def _cfg_from_env_opts(opts: Dict[str, str],
