@@ -1,4 +1,9 @@
+# This file is part of the Open Data Cube, see https://opendatacube.org for more information
+#
+# Copyright (c) 2015-2020 ODC Contributors
+# SPDX-License-Identifier: Apache-2.0
 import warnings
+import toolz
 import rasterio
 from rasterio.shutil import copy as rio_copy
 import numpy as np
@@ -6,7 +11,7 @@ import xarray as xr
 import dask
 from dask.delayed import Delayed
 from pathlib import Path
-from typing import Union, Optional, List, Any
+from typing import Union, Optional, List, Any, Dict
 
 from .io import check_write_path
 from .geometry import GeoBox
@@ -21,17 +26,21 @@ def _adjust_blocksize(block, dim):
     return align_up(block, 16)
 
 
-def _write_cog(pix: np.ndarray,
-               geobox: GeoBox,
-               fname: Union[Path, str],
-               nodata: Optional[float] = None,
-               overwrite: bool = False,
-               blocksize: Optional[int] = None,
-               overview_resampling: Optional[str] = None,
-               overview_levels: Optional[List[int]] = None,
-               ovr_blocksize: Optional[int] = None,
-               **extra_rio_opts) -> Union[Path, bytes]:
-    """Write geo-registered ndarray to GeoTiff file or RAM.
+def _write_cog(
+    pix: np.ndarray,
+    geobox: GeoBox,
+    fname: Union[Path, str],
+    nodata: Optional[float] = None,
+    overwrite: bool = False,
+    blocksize: Optional[int] = None,
+    overview_resampling: Optional[str] = None,
+    overview_levels: Optional[List[int]] = None,
+    ovr_blocksize: Optional[int] = None,
+    use_windowed_writes: bool = False,
+    intermediate_compression: Union[bool, str, Dict[str, Any]] = False,
+    **extra_rio_opts
+) -> Union[Path, bytes]:
+    """Write geo-registered ndarray to a GeoTiff file or RAM.
 
     :param pix: ``xarray.DataArray`` with crs or (ndarray, geobox, nodata) triple
     :param fname:  Output file or ":mem:"
@@ -40,20 +49,22 @@ def _write_cog(pix: np.ndarray,
     :param blocksize: Size of internal tiff tiles (512x512 pixels)
     :param ovr_blocksize: Size of internal tiles in overview images (defaults to blocksize)
     :param overview_resampling: Use this resampling when computing overviews
-    :param overview_levels: List of shrink factors to compute overiews for: [2,4,8,16,32]
+    :param overview_levels: List of shrink factors to compute overviews for: [2,4,8,16,32]
                             to disable overviews supply empty list ``[]``
+    :param use_windowed_writes: Write image block by block (might need this for large images)
+    :param intermediate_compression: Configure compression settings for first pass write, default is no compression
     :param extra_rio_opts: Any other option is passed to ``rasterio.open``
 
     When fname=":mem:" write COG to memory rather than to a file and return it
-    as memoryview object.
+    as a memoryview object.
 
     NOTE: about memory requirements
 
-    This function generates temporary in memory tiff file without compression
+    This function generates a temporary in memory tiff file without compression
     to speed things up. It then adds overviews to this file and only then
     copies it to the final destination with requested compression settings.
-    This is necessary to produce compliant COG, since COG standard demands
-    overviews to be placed before native resolution data and double pass is the
+    This is necessary to produce a compliant COG, since the COG standard demands
+    overviews to be placed before native resolution data and a double pass is the
     only way to achieve this currently.
 
     This means that this function will use about 1.5 to 2 times memory taken by `pix`.
@@ -63,11 +74,17 @@ def _write_cog(pix: np.ndarray,
         blocksize = 512
     if ovr_blocksize is None:
         ovr_blocksize = blocksize
-    if overview_levels is None:
-        overview_levels = [2 ** i for i in range(1, 6)]
     if overview_resampling is None:
         overview_resampling = "nearest"
 
+    # normalise intermediate_compression argument to a dict()
+    if isinstance(intermediate_compression, bool):
+        intermediate_compression = (
+            {"compress": "deflate", "zlevel": 2} if intermediate_compression else {}
+        )
+    elif isinstance(intermediate_compression, str):
+        intermediate_compression = {"compress": intermediate_compression}
+
     if pix.ndim == 2:
         h, w = pix.shape
         nbands = 1
@@ -76,7 +93,7 @@ def _write_cog(pix: np.ndarray,
         if pix.shape[:2] == geobox.shape:
             pix = pix.transpose([2, 0, 1])
         elif pix.shape[-2:] != geobox.shape:
-            raise ValueError('GeoBox shape does not match image shape')
+            raise ValueError("GeoBox shape does not match image shape")
 
         nbands, h, w = pix.shape
         band = tuple(i for i in range(1, nbands + 1))
@@ -85,6 +102,12 @@ def _write_cog(pix: np.ndarray,
 
     assert geobox.shape == (h, w)
 
+    if overview_levels is None:
+        if min(w, h) < 512:
+            overview_levels = []
+        else:
+            overview_levels = [2 ** i for i in range(1, 6)]
+
     if fname != ":mem:":
         path = check_write_path(
             fname, overwrite
@@ -93,7 +116,7 @@ def _write_cog(pix: np.ndarray,
     resampling = rasterio.enums.Resampling[overview_resampling]
 
     if (blocksize % 16) != 0:
-        warnings.warn(f"Block size must be a multiple of 16, will be adjusted")
+        warnings.warn("Block size must be a multiple of 16, will be adjusted")
 
     rio_opts = dict(
         width=w,
@@ -115,28 +138,39 @@ def _write_cog(pix: np.ndarray,
 
     rio_opts.update(extra_rio_opts)
 
+    def _write(pix, band, dst):
+        if not use_windowed_writes:
+            dst.write(pix, band)
+            return
+
+        for _, win in dst.block_windows():
+            if pix.ndim == 2:
+                block = pix[win.toslices()]
+            else:
+                block = pix[(slice(None),) + win.toslices()]
+
+            dst.write(block, indexes=band, window=win)
+
     # Deal efficiently with "no overviews needed case"
     if len(overview_levels) == 0:
         if fname == ":mem:":
             with rasterio.MemoryFile() as mem:
                 with mem.open(driver="GTiff", **rio_opts) as dst:
-                    dst.write(pix, band)
+                    _write(pix, band, dst)
                 return bytes(mem.getbuffer())
         else:
-            with rasterio.open(path, mode='w', driver='GTiff', **rio_opts) as dst:
-                dst.write(pix, band)
+            with rasterio.open(path, mode="w", driver="GTiff", **rio_opts) as dst:
+                _write(pix, band, dst)
             return path
 
     # copy re-compresses anyway so skip compression for temp image
-    tmp_opts = rio_opts.copy()
-    tmp_opts.pop("compress")
-    tmp_opts.pop("predictor")
-    tmp_opts.pop("zlevel")
+    tmp_opts = toolz.dicttoolz.dissoc(rio_opts, "compress", "predictor", "zlevel")
+    tmp_opts.update(intermediate_compression)
 
     with rasterio.Env(GDAL_TIFF_OVR_BLOCKSIZE=ovr_blocksize):
         with rasterio.MemoryFile() as mem:
             with mem.open(driver="GTiff", **tmp_opts) as tmp:
-                tmp.write(pix, band)
+                _write(pix, band, tmp)
                 tmp.build_overviews(overview_levels, resampling)
 
                 if fname == ":mem:":
@@ -146,46 +180,56 @@ def _write_cog(pix: np.ndarray,
                             mem2.name,
                             driver="GTiff",
                             copy_src_overviews=True,
-                            **rio_opts
+                            **toolz.dicttoolz.dissoc(
+                                rio_opts,
+                                "width",
+                                "height",
+                                "count",
+                                "dtype",
+                                "crs",
+                                "transform",
+                                "nodata",
+                            )
                         )
                         return bytes(mem2.getbuffer())
 
-                rio_copy(
-                    tmp, path, driver="GTiff", copy_src_overviews=True, **rio_opts
-                )
+                rio_copy(tmp, path, driver="GTiff", copy_src_overviews=True, **rio_opts)
 
     return path
 
 
 _delayed_write_cog_to_mem = dask.delayed(  # pylint: disable=invalid-name
-    _write_cog,
-    name="compress-cog", pure=True, nout=1
+    _write_cog, name="compress-cog", pure=True, nout=1
 )
 
-_delayed_write_cog_to_file = dask.delayed(   # pylint: disable=invalid-name
-    _write_cog,
-    name="save-cog", pure=False, nout=1
+_delayed_write_cog_to_file = dask.delayed(  # pylint: disable=invalid-name
+    _write_cog, name="save-cog", pure=False, nout=1
 )
 
 
-def write_cog(geo_im: xr.DataArray,
-              fname: Union[str, Path],
-              blocksize: Optional[int] = None,
-              ovr_blocksize: Optional[int] = None,
-              overview_resampling: Optional[str] = None,
-              overview_levels: Optional[List[int]] = None,
-              **extra_rio_opts) -> Union[Path, bytes, Delayed]:
+def write_cog(
+    geo_im: xr.DataArray,
+    fname: Union[str, Path],
+    overwrite: bool = False,
+    blocksize: Optional[int] = None,
+    ovr_blocksize: Optional[int] = None,
+    overview_resampling: Optional[str] = None,
+    overview_levels: Optional[List[int]] = None,
+    use_windowed_writes: bool = False,
+    intermediate_compression: Union[bool, str, Dict[str, Any]] = False,
+    **extra_rio_opts
+) -> Union[Path, bytes, Delayed]:
     """
     Save ``xarray.DataArray`` to a file in Cloud Optimized GeoTiff format.
 
     This function is "Dask aware". If ``geo_im`` is a Dask array, then the
     output of this function is also a Dask Delayed object. This allows us to
     save multiple images concurrently across a Dask cluster. If you are not
-    familiar with Dask this can be confusing, as no operation is performed until
+    familiar with Dask this can be confusing, as no operation is performed until the
     ``.compute()`` method is called, so if you call this function with Dask
     array it will return immediately without writing anything to disk.
 
-    If you are using Dask to speed up data loading, follow example below:
+    If you are using Dask to speed up data loading, follow the example below:
 
     .. code-block:: python
 
@@ -197,11 +241,16 @@ def write_cog(geo_im: xr.DataArray,
 
     :param geo_im: ``xarray.DataArray`` with crs
     :param fname: Output path or ``":mem:"`` in which case compress to RAM and return bytes
+    :param overwrite: True -- replace existing file, False -- abort with IOError exception
     :param blocksize: Size of internal tiff tiles (512x512 pixels)
     :param ovr_blocksize: Size of internal tiles in overview images (defaults to blocksize)
     :param overview_resampling: Use this resampling when computing overviews
     :param overview_levels: List of shrink factors to compute overiews for: [2,4,8,16,32],
                             to disable overviews supply empty list ``[]``
+    :param nodata: Set ``nodata`` flag to this value if supplied, by default ``nodata`` is
+                   read from the attributes of the input array (``geo_im.attrs['nodata']``).
+    :param use_windowed_writes: Write image block by block (might need this for large images)
+    :param intermediate_compression: Configure compression settings for first pass write, default is no compression
     :param extra_rio_opts: Any other option is passed to ``rasterio.open``
 
     :returns: Path to which output was written
@@ -212,18 +261,18 @@ def write_cog(geo_im: xr.DataArray,
 
        **memory requirements**
 
-       This function generates temporary in memory tiff file without
+       This function generates a temporary in memory tiff file without
        compression to speed things up. It then adds overviews to this file and
        only then copies it to the final destination with requested compression
-       settings. This is necessary to produce compliant COG, since COG standard
+       settings. This is necessary to produce a compliant COG, since the COG standard
        demands overviews to be placed before native resolution data and double
        pass is the only way to achieve this currently.
 
        This means that this function will use about 1.5 to 2 times memory taken by ``geo_im``.
     """
     pix = geo_im.data
-    geobox = getattr(geo_im, 'geobox', None)
-    nodata = extra_rio_opts.pop('nodata', None)
+    geobox = getattr(geo_im, "geobox", None)
+    nodata = extra_rio_opts.pop("nodata", None)
     if nodata is None:
         nodata = geo_im.attrs.get("nodata", None)
 
@@ -231,7 +280,11 @@ def write_cog(geo_im: xr.DataArray,
         raise ValueError("Need geo-registered array on input")
 
     if dask.is_dask_collection(pix):
-        real_op = _delayed_write_cog_to_mem if fname == ":mem:" else _delayed_write_cog_to_file
+        real_op = (
+            _delayed_write_cog_to_mem
+            if fname == ":mem:"
+            else _delayed_write_cog_to_file
+        )
     else:
         real_op = _write_cog
 
@@ -240,29 +293,37 @@ def write_cog(geo_im: xr.DataArray,
         geobox,
         fname,
         nodata=nodata,
+        overwrite=overwrite,
         blocksize=blocksize,
         ovr_blocksize=ovr_blocksize,
         overview_resampling=overview_resampling,
         overview_levels=overview_levels,
-        **extra_rio_opts)
+        use_windowed_writes=use_windowed_writes,
+        intermediate_compression=intermediate_compression,
+        **extra_rio_opts
+    )
 
 
-def to_cog(geo_im: xr.DataArray,
-           blocksize: Optional[int] = None,
-           ovr_blocksize: Optional[int] = None,
-           overview_resampling: Optional[str] = None,
-           overview_levels: Optional[List[int]] = None,
-           **extra_rio_opts) -> Union[bytes, Delayed]:
+def to_cog(
+    geo_im: xr.DataArray,
+    blocksize: Optional[int] = None,
+    ovr_blocksize: Optional[int] = None,
+    overview_resampling: Optional[str] = None,
+    overview_levels: Optional[List[int]] = None,
+    use_windowed_writes: bool = False,
+    intermediate_compression: Union[bool, str, Dict[str, Any]] = False,
+    **extra_rio_opts
+) -> Union[bytes, Delayed]:
     """
     Compress ``xarray.DataArray`` into Cloud Optimized GeoTiff bytes in memory.
 
-    This function doesn't write to disk, it compresses in RAM, this is useful
+    This function doesn't write to disk, it compresses in RAM, which is useful
     for saving data to S3 or other cloud object stores.
 
     This function is "Dask aware". If ``geo_im`` is a Dask array, then the
     output of this function is also a Dask Delayed object. This allows us to
     compress multiple images concurrently across a Dask cluster. If you are not
-    familiar with Dask this can be confusing, as no operation is performed until
+    familiar with Dask this can be confusing, as no operation is performed until the
     ``.compute()`` method is called, so if you call this function with Dask
     array it will return immediately without compressing any data.
 
@@ -271,6 +332,10 @@ def to_cog(geo_im: xr.DataArray,
     :param ovr_blocksize: Size of internal tiles in overview images (defaults to blocksize)
     :param overview_resampling: Use this resampling when computing overviews
     :param overview_levels: List of shrink factors to compute overiews for: [2,4,8,16,32]
+    :param nodata: Set ``nodata`` flag to this value if supplied, by default ``nodata`` is
+                   read from the attributes of the input array (``geo_im.attrs['nodata']``).
+    :param use_windowed_writes: Write image block by block (might need this for large images)
+    :param intermediate_compression: Configure compression settings for first pass write, default is no compression
     :param extra_rio_opts: Any other option is passed to ``rasterio.open``
 
     :returns: In-memory GeoTiff file as bytes
@@ -279,13 +344,19 @@ def to_cog(geo_im: xr.DataArray,
     Also see :py:meth:`~datacube.utils.cog.write_cog`
 
     """
-    bb = write_cog(geo_im,
-                   ":mem:",
-                   blocksize=blocksize,
-                   ovr_blocksize=ovr_blocksize,
-                   overview_resampling=overview_resampling,
-                   overview_levels=overview_levels,
-                   **extra_rio_opts)
+    bb = write_cog(
+        geo_im,
+        ":mem:",
+        blocksize=blocksize,
+        ovr_blocksize=ovr_blocksize,
+        overview_resampling=overview_resampling,
+        overview_levels=overview_levels,
+        use_windowed_writes=use_windowed_writes,
+        intermediate_compression=intermediate_compression,
+        **extra_rio_opts
+    )
 
-    assert isinstance(bb, (bytes, Delayed))  # for mypy sake for :mem: output it bytes or delayed bytes
+    assert isinstance(
+        bb, (bytes, Delayed)
+    )  # for mypy sake for :mem: output it bytes or delayed bytes
     return bb
