@@ -1,10 +1,14 @@
+# This file is part of the Open Data Cube, see https://opendatacube.org for more information
+#
+# Copyright (c) 2015-2020 ODC Contributors
+# SPDX-License-Identifier: Apache-2.0
 from collections import OrderedDict
 from datetime import datetime
 from copy import deepcopy
 import warnings
 
 import pytest
-import mock
+from unittest import mock
 import numpy
 
 from datacube.model import DatasetType, MetadataType, Dataset, GridSpec
@@ -12,11 +16,29 @@ from datacube.utils import geometry
 from datacube.virtual import construct_from_yaml, catalog_from_yaml, VirtualProductException
 from datacube.virtual import DEFAULT_RESOLVER, Transformation
 from datacube.virtual.impl import Datacube
+from datacube.virtual.expr import formula_parser, FormulaEvaluator, evaluate_data
 
 
 ##########################################
 # Set up some common test data and fixtures
 
+
+def test_formula_parsing():
+    parser = formula_parser()
+    evaluator = FormulaEvaluator
+    env = dict(x=4, y=2, true=True, false=False)
+
+    assert evaluate_data('x', env, parser, evaluator) == 4
+    assert evaluate_data('-y', env, parser, evaluator) == -2
+    assert evaluate_data('x / y', env, parser, evaluator) == 2
+    assert evaluate_data('(x + y) * 3', env, parser, evaluator) == 18
+    assert not evaluate_data('x == y', env, parser, evaluator)
+    assert not evaluate_data('true == false', env, parser, evaluator)
+    assert evaluate_data('not (true == false)', env, parser, evaluator)
+    assert evaluate_data('x > y', env, parser, evaluator)
+    assert not evaluate_data('(x > y) & (x < y)', env, parser, evaluator)
+
+
 PRODUCT_LIST = ['ls7_pq_albers', 'ls8_pq_albers', 'ls7_nbar_albers', 'ls8_nbar_albers']
 
 
@@ -156,6 +178,18 @@ def catalog():
                                 measurements: [blue]
                               - product: ls8_nbar_albers
                                 measurements: [blue]
+
+            reproject_utm:
+                recipe:
+                    reproject:
+                        output_crs: EPSG:32755
+                        resolution: [30, -30]
+                    input:
+                        collate:
+                          - product: ls7_nbar_albers
+                            measurements: [blue]
+                          - product: ls8_nbar_albers
+                            measurements: [blue]
     """)
 
 
@@ -195,7 +229,7 @@ def dc():
         result.center_time = center_time
         return result
 
-    def search(*args, **kwargs):
+    def find_datasets(*args, **kwargs):
         product = kwargs['product']
         if product == 'ls8_nbar_albers':
             return [example_dataset(product, ids[0], datetime(2014, 2, 7, 23, 57, 26)),
@@ -211,7 +245,7 @@ def dc():
 
     result.index.products.get_all = lambda: [example_product(x) for x in PRODUCT_LIST]
     result.index.products.get_by_name = example_product
-    result.index.datasets.search = search
+    result.find_datasets = find_datasets
     return result
 
 
@@ -487,3 +521,17 @@ def test_register(dc, query):
         data = bluegreen.load(dc, **query)
 
     assert 'bluegreen' in data
+
+
+def test_reproject(dc, query, catalog):
+    reproject_utm = catalog['reproject_utm']
+
+    with mock.patch('datacube.virtual.impl.Datacube') as mock_datacube:
+        mock_datacube.load_data = load_data
+        mock_datacube.group_datasets = group_datasets
+        data = reproject_utm.load(dc, **query)
+
+    assert data.crs == geometry.CRS('EPSG:32755')
+    assert data.geobox.crs == geometry.CRS('EPSG:32755')
+    assert data.coords['x'].attrs['resolution'] == -30
+    assert data.coords['y'].attrs['resolution'] == 30
