@@ -1,3 +1,7 @@
+# This file is part of the Open Data Cube, see https://opendatacube.org for more information
+#
+# Copyright (c) 2015-2020 ODC Contributors
+# SPDX-License-Identifier: Apache-2.0
 """
 Test utility functions from :module:`datacube.utils`
 
@@ -359,7 +363,7 @@ A:..:0
     assert set(fv.keys()) == set('ABCDE')
     assert isinstance(dg, list)
     assert len(dg) == 4
-    assert [len(l) for l in dg] == [1, 3, 2, 1]
+    assert [len(dss) for dss in dg] == [1, 3, 2, 1]
 
     def to_set(xx):
         return set(x.id for x in xx)
